"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
require("./rmSync-polyfill");
const mime_1 = __importDefault(require("mime"));
const path_1 = __importDefault(require("path"));
const core_shared_1 = require("@vue-image-modernizer/core-shared");
const core_vue2_1 = require("@vue-image-modernizer/core-vue2");
const core_vue3_1 = require("@vue-image-modernizer/core-vue3");
// @ts-expect-error @vue/cli-shared-utils doesn't have typing
const cli_shared_utils_1 = require("@vue/cli-shared-utils");
// need this plugin to remove thread-loader when using vue-loader,
// thread-loader does JSON.stringify to vue-loader's options
// which turns functions to `null`, including our nodeTransform function
class VueImageModernizerWebpackPlugin {
    apply(compiler) {
        const id = "vue-image-modernizer-plugin";
        function normalModuleLoaderHook(loaderContext) {
            const vueLoader = loaderContext._module.loaders.find(({ loader }) => loader.includes("vue-loader"));
            if ((vueLoader === null || vueLoader === void 0 ? void 0 : vueLoader.options) !== undefined)
                loaderContext._module.loaders = loaderContext._module.loaders.filter(({ loader }) => !loader.includes("thread-loader"));
        }
        compiler.hooks.compilation.tap(id, (compilation) => {
            const NormalModule = require("webpack/lib/NormalModule");
            // in webpack 5 compilation.hooks.normalModuleLoader became
            // NormalModule.getCompilationHooks(compilation).loader
            if (NormalModule === null || NormalModule === void 0 ? void 0 : NormalModule.getCompilationHooks)
                NormalModule.getCompilationHooks(compilation).loader.tap(id, normalModuleLoaderHook);
            else
                compilation.hooks.normalModuleLoader.tap(id, normalModuleLoaderHook);
        });
    }
}
/** delete all folders in cacheDirectory with names ending in "-loader", except for those in exceptions */
function deleteLoaderCache(cacheDirectory, exceptions) {
    if (fs_1.default.existsSync(cacheDirectory)) {
        fs_1.default.readdirSync(cacheDirectory, { withFileTypes: true })
            .filter((dirent) => dirent.isDirectory())
            .filter((dirent) => dirent.name.endsWith("-loader"))
            .forEach((dirent) => {
            const cacheFolderPath = path_1.default.join(cacheDirectory, dirent.name);
            if (!exceptions.includes(cacheFolderPath)) {
                fs_1.default.rmSync(cacheFolderPath, { recursive: true, force: true });
            }
        });
    }
}
/** write cacheIdentifier to vimCachePath */
function writeVimCache(vimCachePath, data) {
    const dirPath = path_1.default.dirname(vimCachePath);
    if (!fs_1.default.existsSync(dirPath))
        fs_1.default.mkdirSync(dirPath);
    fs_1.default.writeFileSync(vimCachePath, JSON.stringify(data));
}
function default_1(api, vueCliOptions) {
    var _a, _b, _c, _d, _e;
    const vue = cli_shared_utils_1.loadModule("vue", api.getCwd());
    const isVue3 = vue && cli_shared_utils_1.semver.major(vue.version) === 3;
    const cacheDirectory = api.resolve("node_modules/.cache/");
    const vimCachePath = api.resolve("node_modules/.cache/vue-image-modernizer.json");
    // generate our own cacheIdentifier with our dependencies
    const { cacheIdentifier } = api.genCacheConfig("vue-image-modernizer", {
        "webpack-image-resize-loader": (_a = require("webpack-image-resize-loader/package.json")) === null || _a === void 0 ? void 0 : _a.version,
        "webpack-image-srcset-loader": (_b = require("webpack-image-srcset-loader/package.json")) === null || _b === void 0 ? void 0 : _b.version,
        "@vue-image-modernizer/core-vue3": (_c = require("@vue-image-modernizer/core-vue3/package.json")) === null || _c === void 0 ? void 0 : _c.version,
        "@vue-image-modernizer/vue-cli-plugin-image-modernizer": (_d = require("@vue-image-modernizer/vue-cli-plugin-image-modernizer/package.json")) === null || _d === void 0 ? void 0 : _d.version,
        config: (_e = vueCliOptions.pluginOptions) === null || _e === void 0 ? void 0 : _e.imageModernizer,
    });
    // if cacheIdentifier has changed or doesn't exist, delete loader caches, then write our cacheIdentifier
    if (fs_1.default.existsSync(vimCachePath)) {
        const vimCacheContent = fs_1.default.readFileSync(vimCachePath).toString();
        const cachedCacheIdentifier = JSON.parse(vimCacheContent).cacheIdentifier;
        if (cachedCacheIdentifier !== cacheIdentifier) {
            deleteLoaderCache(cacheDirectory, [vimCachePath]);
            writeVimCache(vimCachePath, { cacheIdentifier });
        }
    }
    else {
        deleteLoaderCache(cacheDirectory, [vimCachePath]);
        writeVimCache(vimCachePath, { cacheIdentifier });
    }
    api.chainWebpack((config) => {
        var _a;
        const builtInImageLoader = config.module
            .rule("images")
            .use("url-loader")
            .get("loader");
        const builtInImageLoaderEscaped = builtInImageLoader
            .replace(/\\/g, "\\\\")
            .replace(/!/g, "\\x21");
        const builtInImageLoaderOptionsString = JSON.stringify(config.module.rule("images").use("url-loader").get("options"));
        const builtInImageLoaderOptionsStringEscaped = builtInImageLoaderOptionsString
            .replace(/\\/g, "\\\\")
            .replace(/!/g, "\\x21");
        const fileLoaderOptionsGenerator = `(options, existingOptions) => ({
      ...existingOptions,
      mimetype: require(\`mime\`).getType(options.format),
      fallback: {
        ...existingOptions.fallback,
        options: defaultFileLoaderOptionsGenerator(
          options,
          existingOptions.fallback.options
        ),
      },
    })`.replace(/\s/g, "");
        function compressFilePathTransformer(path, options, type) {
            var _a, _b;
            const imageMIMEType = mime_1.default.getType(path);
            const targetMIMEType = type !== null && type !== void 0 ? type : imageMIMEType;
            const format = core_shared_1.IMAGE_FORMATS[targetMIMEType];
            const resizeLoaderOptions = {
                format: imageMIMEType === type ? undefined : format,
                quality: options.quality[core_shared_1.IMAGE_FORMATS[targetMIMEType]],
                fileLoader: "url-loader",
                fileLoaderOptionsGenerator,
                ...(_b = (_a = vueCliOptions.pluginOptions) === null || _a === void 0 ? void 0 : _a.imageModernizer) === null || _b === void 0 ? void 0 : _b.imageResizeLoaderOptions,
            };
            const resizeLoaderOptionsString = JSON.stringify(resizeLoaderOptions);
            return `-!${builtInImageLoaderEscaped}?${builtInImageLoaderOptionsStringEscaped}!webpack-image-resize-loader?${resizeLoaderOptionsString}!${path}`;
        }
        function srcSetFilePathTransformer(path, options, type) {
            var _a, _b, _c, _d;
            const imageMIMEType = mime_1.default.getType(path);
            const targetMIMEType = type !== null && type !== void 0 ? type : imageMIMEType;
            const format = core_shared_1.IMAGE_FORMATS[targetMIMEType];
            const resizeLoaderOptions = {
                format: imageMIMEType === type ? undefined : format,
                quality: options.quality[core_shared_1.IMAGE_FORMATS[targetMIMEType]],
                fileLoader: "url-loader",
                fileLoaderOptionsGenerator,
                ...(_b = (_a = vueCliOptions.pluginOptions) === null || _a === void 0 ? void 0 : _a.imageModernizer) === null || _b === void 0 ? void 0 : _b.imageResizeLoaderOptions,
            };
            const resizeLoaderOptionsString = JSON.stringify(resizeLoaderOptions);
            const srcsetLoaderOptions = {
                sizes: options.sizes,
                ...(_d = (_c = vueCliOptions.pluginOptions) === null || _c === void 0 ? void 0 : _c.imageModernizer) === null || _d === void 0 ? void 0 : _d.imageSrcsetLoaderOptions,
            };
            const srcsetLoaderOptionsString = JSON.stringify(srcsetLoaderOptions);
            return `-!webpack-image-srcset-loader?${srcsetLoaderOptionsString}!${builtInImageLoaderEscaped}?${builtInImageLoaderOptionsStringEscaped}!webpack-image-resize-loader?${resizeLoaderOptionsString}!${path}`;
        }
        const vimOptions = {
            compressFilePathTransformer,
            srcSetFilePathTransformer,
            ...(_a = vueCliOptions.pluginOptions) === null || _a === void 0 ? void 0 : _a.imageModernizer,
        };
        if (isVue3) {
            config.module
                .rule("vue")
                .test(/\.vue$/)
                .use("vue-loader")
                .tap((options) => {
                var _a, _b, _c;
                return ({
                    ...options,
                    // concat the original cacheIdentifier with our cacheIdentifier
                    cacheIdentifier: ((_a = options.cacheIdentifier) !== null && _a !== void 0 ? _a : "") + cacheIdentifier,
                    compilerOptions: {
                        ...options === null || options === void 0 ? void 0 : options.compilerOptions,
                        nodeTransforms: [
                            ...((_c = (_b = options === null || options === void 0 ? void 0 : options.compilerOptions) === null || _b === void 0 ? void 0 : _b.nodeTransforms) !== null && _c !== void 0 ? _c : []),
                            core_vue3_1.createVIMNodeTransformWithOptions(vimOptions),
                        ],
                    },
                });
            });
        }
        else {
            config.module
                .rule("vue")
                .test(/\.vue$/)
                .use("vue-loader")
                .tap((options) => {
                var _a, _b, _c;
                return ({
                    ...options,
                    // concat the original cacheIdentifier with our cacheIdentifier
                    cacheIdentifier: ((_a = options.cacheIdentifier) !== null && _a !== void 0 ? _a : "") + cacheIdentifier,
                    compilerOptions: {
                        ...options === null || options === void 0 ? void 0 : options.compilerOptions,
                        modules: [
                            ...((_c = (_b = options === null || options === void 0 ? void 0 : options.compilerOptions) === null || _b === void 0 ? void 0 : _b.modules) !== null && _c !== void 0 ? _c : []),
                            core_vue2_1.createVIMModuleWithOptions(vimOptions),
                        ],
                    },
                });
            });
        }
        config.plugin("vue-image-modernizer").use(VueImageModernizerWebpackPlugin);
    });
}
exports.default = default_1;
//# sourceMappingURL=index.js.map